module full_adder(
    input a,b,
    input cin,
    output s,
    output cout
    );

assign s = a ^ b ^ cin;
assign cout = (a & b) | (a & cin) | (b & cin);

endmodule

module adder4( 	
input [3:0] A,B,
input cin,
output [3:0] S,
output cout
); 

wire [2:0] carry;

  full_adder fa0(A[0], B[0],      cin, S[0], carry[0]);
  full_adder fa1(A[1], B[1], carry[0], S[1], carry[1]);
  full_adder fa2(A[2], B[2], carry[1], S[2], carry[2]);
  full_adder fa3(A[3], B[3], carry[2], S[3], cout);

endmodule 

module adder16( 	
input [15:0] A,B,
input cin,
output [15:0] S,
output cout
); 

wire [2:0] carry;

  adder4 A0(A[3:0],  B[3:0],      cin,  S[3:0],  carry[0]);
  adder4 A1(A[7:4],  B[7:4],  carry[0], S[7:4],  carry[1]);
  adder4 A2(A[11:8], B[11:8], carry[1], S[11:8], carry[2]);
  adder4 A3(A[15:12],B[15:12],carry[2], S[15:12],cout);

endmodule 


module main;

reg [15:0] a,b;
reg cin;
wire [15:0] s;
wire cout;
adder16 sumator(a, b, cin, s, cout);

initial begin
$display ("time\t  a   |    b|c_in |s     |c_out");
$display ("________________________________________");
$monitor ("%g\t     %d|%d|%b    |%d |%b", $time, a, b, cin, s, cout);

a=0;
b=0;
cin=0;
#10

a=3245;
b=2468;
cin=0;
#10

a=1234;
b=34287;
cin=1;
#10

a=65535;
b=0;
cin=1;
#10

a=1234;
b=4321;
cin=0;
#10

a=5;
b=10;
cin=1;
#10

$finish;

end
endmodule