module full_adder(
    input a,b,
    input cin,
    output s,
    output cout
    );

assign s = a ^ b ^ cin;
assign cout = (a & b) | (a & cin) | (b & cin);

endmodule

module adder_param 
( 	
input [width-1:0] A,B,
input cin,
output [width-1:0] S,
output cout
); 

parameter width = 32;

wire [width:0] carry;
assign carry[0] = cin;
assign cout = carry[width];

genvar i;
    generate
        for(i = 0; i < width; i= i + 1)
            begin : RIPPLE
                full_adder FA(A[i], B[i], carry[i], S[i], carry[i+1]);
            end
    endgenerate
  
endmodule 
 
module main;

reg [31:0] a,b;
reg cin;
wire [31:0] s;
wire cout;
adder_param sumator(a, b, cin, s, cout);

initial begin
$display ("time\t  a   |         b|c_in |s          |c_out");
$display ("________________________________________");
$monitor ("%g\t%d|%d|%b    |%d |%b", $time, a, b, cin, s, cout);

a=0;
b=0;
cin=0;
#10

a=32450123;
b=24536467;
cin=0;
#10

a=123456789;
b=987654321;
cin=0;
#10

a=123456789;
b=987654321;
cin=1;
#10

a=4294967295;
b=1;
cin=0;
#10

a=4294967295;
b=0;
cin=1;
#10

$finish;

end
endmodule